
#include "defaultgui.h"
#include "helper.h"
#include "core.h"
#include "global.h"
#include "timeslider.h"
#include "constants.h"

#include <qmainwindow.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include "myaction.h"
#include <qtooltip.h>
#include <qpushbutton.h>


DefaultGui::DefaultGui( QString file, QWidget * parent, const char* name, 
                        WFlags fl )
	: BaseGui( file, parent, name, fl )
{
	createStatusBar();

	connect( this, SIGNAL(timeChanged(double, int, QString)),
             this, SLOT(displayTime(double, int, QString)) );
    connect( this, SIGNAL(frameChanged(int)),
             this, SLOT(displayFrame(int)) );

	connect( this, SIGNAL(cursorNearBottom(QPoint)), 
             this, SLOT(showFloatingControl(QPoint)) );
	connect( this, SIGNAL(cursorNearTop(QPoint)), 
             this, SLOT(showFloatingMenu(QPoint)) );
	connect( this, SIGNAL(cursorFarEdges()), 
             this, SLOT(hideFloatingControls()) );

	createToolBars();

	languageChange();
}

DefaultGui::~DefaultGui() {
}

void DefaultGui::createToolBars() {
	controlwidget = new QToolBar( this, "controlwidget" );
	controlwidget->setResizeEnabled(TRUE);
	addDockWindow(controlwidget, Qt::DockBottom );

	rewind3Act->addTo( controlwidget );
	rewind2Act->addTo( controlwidget );
	rewind1Act->addTo( controlwidget );

	timeslider = new TimeSlider( controlwidget, "timeslider");
	timeslider->setFocusPolicy( NOFOCUS );
	timeslider->setOrientation( Qt::Horizontal );
	timeslider->setSizePolicy( QSizePolicy::Expanding , QSizePolicy::Expanding );
	connect( timeslider, SIGNAL( posChanged(int) ), core, SLOT(goToPos(int)) );
	connect( timeslider, SIGNAL( draggingPos(int) ), this, SLOT(displayGotoTime(int)) );
	controlwidget->setStretchableWidget( timeslider );

	forward1Act->addTo( controlwidget );
	forward2Act->addTo( controlwidget );
	forward3Act->addTo( controlwidget );


	controlwidget2 = new QToolBar( this, "controlwidget2" );
	controlwidget2->setResizeEnabled(TRUE);
	addDockWindow(controlwidget2, Qt::DockBottom );

	controlwidget2->setNewLine(TRUE);
	playAct->addTo( controlwidget2 );
	pauseAct->addTo( controlwidget2 );
	stopAct->addTo( controlwidget2 );

	QLabel * spacer = new QLabel( controlwidget2 );
	spacer->setSizePolicy( QSizePolicy::Expanding , QSizePolicy::Expanding );

	fullscreenAct->addTo( controlwidget2 );
	muteAct->addTo( controlwidget2 );

	volumeslider = new MySlider( controlwidget2, "volumeslider" );
	volumeslider->setOrientation( Qt::Horizontal );
	volumeslider->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
	volumeslider->setFocusPolicy( NOFOCUS );
	volumeslider->setTickmarks( QSlider::Below );
	volumeslider->setTickInterval( 10 );
	volumeslider->setLineStep( 1 );
	volumeslider->setPageStep( 10 );
	connect( volumeslider, SIGNAL( valueChanged(int) ), 
             core, SLOT( setVolume(int) ) );

	toolbar = new QToolBar( this, "toolbar" );
	openFileAct->addTo( toolbar );
	openDVDAct->addTo( toolbar );
	openURLAct->addTo( toolbar );
	toolbar->addSeparator();
	compactAct->addTo( toolbar );
	toolbar->addSeparator();
	screenshotAct->addTo( toolbar );
	toolbar->addSeparator();
	showInfoAct->addTo( toolbar );
	showPlaylistAct->addTo( toolbar );
	showPreferencesAct->addTo( toolbar );

	toolbar2 = new QToolBar( this, "toolbar2" );
	select_audio = new QPushButton( toolbar2, "select_audio" );
	select_audio->setPopup( (QPopupMenu *) audiotrack_menu );

	select_subtitle = new QPushButton( toolbar2, "select_subtitle" );
	select_subtitle->setPopup( (QPopupMenu *) subtitlestrack_menu );

	controlwidget->show();
	controlwidget2->show();
	toolbar->show();
	toolbar2->show();
}

void DefaultGui::createStatusBar() {
	qDebug("DefaultGui::createStatusBar");

	time_display = new QLabel( statusBar() );
	time_display->setAlignment(Qt::AlignRight);

	frame_display = new QLabel( statusBar() );
	frame_display->setAlignment(Qt::AlignRight);
    #if QT_VERSION >= 0x040000
	statusBar()->setAutoFillBackground(TRUE);
	#endif
	statusBar()->setPaletteBackgroundColor( QColor(0,0,0) );
	statusBar()->setPaletteForegroundColor( QColor(255,255,255) );
	time_display->setPaletteBackgroundColor( QColor(0,0,0) );
	time_display->setPaletteForegroundColor( QColor(255,255,255) );
	frame_display->setPaletteBackgroundColor( QColor(0,0,0) );
	frame_display->setPaletteForegroundColor( QColor(255,255,255) );
	statusBar()->setSizeGripEnabled(FALSE);

    statusBar()->message( tr("Welcome to SMPlayer") );
	statusBar()->addWidget( frame_display, 0, TRUE );
	frame_display->setText( " 0 " );

    statusBar()->addWidget( time_display, 0, TRUE );
	time_display->setText("00:00:00 / 00:00:00");

	time_display->show();
	frame_display->hide();
}

void DefaultGui::languageChange() {
	BaseGui::languageChange();

	QToolTip::add( volumeslider, tr("Volume") );

	select_audio->setText( tr("Audio") );
	select_subtitle->setText( tr("Subtitle") );
}

void DefaultGui::displayTime(double sec, int perc, QString text) {
	time_display->setText( text );
	timeslider->setPos(perc);
}

void DefaultGui::displayFrame(int frame) {
	//qDebug("DefaultGui::displayFrame: %d", frame);

	if (frame_display->isVisible()) {
		frame_display->setNum( frame );
	}
}

void DefaultGui::updateWidgets() {
	qDebug("DefaultGui::updateWidgets");

	BaseGui::updateWidgets();

	// Frame counter
	frame_display->setShown( pref->show_frame_counter );

	// Volume slider
	volumeslider->setValue( core->mset.volume );
}

void DefaultGui::aboutToEnterFullscreen() {
	qDebug("DefaultGui::aboutToEnterFullscreen");

	if (!pref->compact_mode) {
		menuBar()->hide();
		statusBar()->hide();
		controlwidget->hide();
		controlwidget2->hide();
		toolbar->hide();
		toolbar2->hide();
		#if DOCK_PLAYLIST
		playlistdock->hide();
		#endif
	}

}

void DefaultGui::aboutToExitFullscreen() {
	qDebug("DefaultGui::aboutToExitFullscreen");

	if (!pref->compact_mode) {
		menuBar()->show();
		statusBar()->show();
		controlwidget->show();
		controlwidget2->show();
		toolbar->show();
		toolbar2->show();
		#if DOCK_PLAYLIST
		playlistdock->show();
		#endif
	}

}

void DefaultGui::aboutToEnterCompactMode() {
	menuBar()->hide();
	statusBar()->hide();
	controlwidget->hide();
	controlwidget2->hide();
	toolbar->hide();
	toolbar2->hide();
}

void DefaultGui::aboutToExitCompactMode() {
	menuBar()->show();
	statusBar()->show();
	controlwidget->show();
	controlwidget2->show();
	toolbar->show();
	toolbar2->show();
}

void DefaultGui::showFloatingControl(QPoint p) {
	qDebug("DefaultGui::showFloatingControl");

	if (!controlwidget->isVisible())	
		controlwidget->show();

	if (!controlwidget2->isVisible())	
		controlwidget2->show();

	/*
	//if (controlwidget->place() == QDockWindow::InDock) {
	if (!controlwidget->isVisible()) {
		controlwidget->undock();
		controlwidget->show();
	}
	*/
}

void DefaultGui::showFloatingMenu(QPoint p) {
	qDebug("DefaultGui::showFloatingMenu");

	if (!menuBar()->isVisible())
		menuBar()->show();
}

void DefaultGui::hideFloatingControls() {
	qDebug("DefaultGui::hideFloatingControls");

	if (controlwidget->isVisible())	
		controlwidget->hide();

	if (controlwidget2->isVisible())	
		controlwidget2->hide();

	if (menuBar()->isVisible())	
		menuBar()->hide();

	/*
	//if (controlwidget->place() == QDockWindow::OutsideDock) {
	if (controlwidget->isVisible()) {
		controlwidget->dock();
		controlwidget->hide();
	}
	*/
}
